package gov.usgs.earthquake.nshmp.calc;

import gov.usgs.earthquake.nshmp.gmm.Gmm;
import gov.usgs.earthquake.nshmp.model.SourceType;

/**
 * Data type identifiers. These are used to specify the different types of
 * hazard curves or other data that should be saved after a calculation is
 * complete.
 *
 * @author U.S. Geological Survey
 */
public enum DataType {

  /** Total hazard curves or magnitude-frequency distributions, etc. */
  TOTAL,

  /** Results grouped by {@linkplain Gmm Ground motion model}. */
  GMM,

  /** Results decomposed by magnitude. */
  MAGNITUDE,

  /** Map datasets at specified return periods. */
  MAP,

  /** Results grouped by {@link SourceType}. */
  SOURCE,

  /** Uncertainties. */
  UNCERTAINTY,

  /** Disaggregation r-m-ε bin data */
  DISAGG_DATA;

}
