package gov.usgs.earthquake.nshmp.geo;

import static com.google.common.base.Preconditions.checkState;

import java.util.AbstractList;
import java.util.Arrays;
import java.util.Objects;

/**
 * Default LocationList implementation backed by an array.
 *
 * @author U.S. Geological Survey
 */
final class RegularLocationList extends AbstractList<Location> implements LocationList {

  private final Location[] locations;

  RegularLocationList(Location[] locations) {
    checkState(locations.length > 0, "Location lists may not by empty");
    Arrays.stream(locations).forEach(Objects::requireNonNull);
    this.locations = locations;
  }

  @Override
  public Location get(int index) {
    return locations[index];
  }

  @Override
  public int size() {
    return locations.length;
  }

  @Override
  public boolean equals(Object obj) {
    return super.equals(obj) && (obj instanceof LocationList);
  }

  @Override
  public String toString() {
    return Locations.toString(this);
  }
}
