package gov.usgs.earthquake.nshmp.model;

import gov.usgs.earthquake.nshmp.fault.surface.RuptureSurface;

/**
 * A {@code Rupture} is a proxy for an actual earthquake and encapsulates all
 * the source information required by a ground motion model (Gmm).
 *
 * @author U.S. Geological Survey
 */
public class RegularRupture implements Rupture {

  private final double mag;
  private final double rake;
  private final double rate;
  private final RuptureSurface surface;

  public RegularRupture(double mag, double rate, double rake, RuptureSurface surface) {
    this.mag = mag;
    this.rate = rate;
    this.rake = rake;
    this.surface = surface;
  }

  @Override
  public double mag() {
    return mag;
  }

  @Override
  public double rake() {
    return rake;
  }

  @Override
  public double rate() {
    return rate;
  }

  @Override
  public RuptureSurface surface() {
    return surface;
  }

  @Override
  public String toString() {
    return Rupture.asString(this);
  }

}
