package gov.usgs.earthquake.nshmp.model;

import java.util.Map;

import gov.usgs.earthquake.nshmp.fault.surface.RuptureSurface;

/**
 * A {@code Rupture} is a proxy for an actual earthquake and encapsulates all
 * the source information required by a ground motion model (Gmm).
 *
 * @author U.S. Geological Survey
 */
public interface Rupture {

  /**
   * The {@code Rupture} magnitude.
   */
  double mag();

  /**
   * The {@code Rupture} rake.
   */
  double rake();

  /**
   * The {@code Rupture} surface.
   */
  RuptureSurface surface();

  /**
   * The {@code Rupture} rate.
   */
  double rate();

  /**
   * toString() helper for implementations.
   */
  static String asString(Rupture rupture) {
    Map<String, ?> data = Map.of(
        "mag", rupture.mag(),
        "rake", rupture.rake(),
        "rate", rupture.rate(),
        "surface", rupture.surface());
    return rupture.getClass().getSimpleName() + " " + data;
  }

}
