package gov.usgs.earthquake.nshmp.model;

import java.util.List;

import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.mfd.Mfd;

/**
 * An earthquake {@code Source}; usually some physical or pseudo-representation
 * of a fault and associated {@link Mfd}s governing the size and rate of all
 * possible {@link Rupture}s.
 *
 * @author U.S. Geological Survey
 */
public interface Source extends List<Rupture> {

  /**
   * The source name.
   */
  String name();

  /**
   * The source unique numeric identifier.
   */
  int id();

  /**
   * The source type.
   */
  SourceType type();

  /**
   * The {@code Location} of this source relative to the supplied {@code site}
   * location. The details of what this method returns are implementation
   * secific.
   */
  Location location(Location site);

  /**
   * The MFDs that define earthquake rates for this source.
   */
  List<Mfd> mfds(); // this may not really be needed

}
