package gov.usgs.earthquake.nshmp.model;

import static gov.usgs.earthquake.nshmp.model.SourceType.DECOLLEMENT;
import static gov.usgs.earthquake.nshmp.model.SourceType.FAULT;
import static gov.usgs.earthquake.nshmp.model.SourceType.GRID;
import static gov.usgs.earthquake.nshmp.model.SourceType.INTERFACE;
import static gov.usgs.earthquake.nshmp.model.SourceType.INTERFACE_GRID;
import static gov.usgs.earthquake.nshmp.model.SourceType.SLAB;
import static gov.usgs.earthquake.nshmp.model.SourceType.ZONE;

import java.util.EnumSet;
import java.util.Set;

import gov.usgs.earthquake.nshmp.Text;

/**
 * Tectonic setting identifier. These are used to make associations between
 * earthquake sources and ground motion models specific to each setting.
 *
 * @author U.S. Geological Survey
 */
public enum TectonicSetting {

  /** Active, shallow crust. */
  ACTIVE_CRUST,

  /** Stable, shallow crust. */
  STABLE_CRUST,

  /** Subduction zone. */
  SUBDUCTION,

  /** Volcanic. */
  VOLCANIC;

  @Override
  public String toString() {
    return Text.enumLabelWithSpaces(this, true);
  }

  /**
   * Return the set of top level source types supported in this tectonic
   * setting.
   */
  public Set<SourceType> sourceTypes() {
    switch (this) {
      case ACTIVE_CRUST:
        return EnumSet.of(FAULT, GRID, ZONE);
      case STABLE_CRUST:
        return EnumSet.of(FAULT, GRID, ZONE);
      case SUBDUCTION:
        return EnumSet.of(INTERFACE, INTERFACE_GRID, SLAB);
      case VOLCANIC:
        return EnumSet.of(DECOLLEMENT, FAULT, GRID, ZONE);
      default:
        throw new UnsupportedOperationException();
    }
  }
}
