package gov.usgs.earthquake.nshmp.calc;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.OptionalDouble;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.model.SiteData;

class SitesTests {

  static final Path RESOURCE_PATH = Paths.get("src/test/resources/calc");
  static final Path CSV_PATH = RESOURCE_PATH.resolve("sites.csv");
  static final Path CSV_NO_PROPS_PATH = RESOURCE_PATH.resolve("sites-no-props.csv");
  static final Path GEOJSON_POINTS_PATH = RESOURCE_PATH.resolve("sites-points.geojson");
  static final Path GEOJSON_POLYGON_PATH = RESOURCE_PATH.resolve("sites-polygon.geojson");

  @Test
  final void testCsv() throws IOException {

    List<Site> sites = Sites.fromCsv(
        CSV_PATH, SiteData.EMPTY, OptionalDouble.empty());
    assertEquals(6, sites.size());

    List<Site> sitesNoProps = Sites.fromCsv(
        CSV_NO_PROPS_PATH, SiteData.EMPTY, OptionalDouble.empty());
    assertEquals(6, sitesNoProps.size());

    // toString
    assertEquals(TO_STRING, Sites.toString(sites));
  }

  @Test
  final void testGeoJson() throws IOException {

    List<Site> sitesPoints = Sites.fromGeoJson(
        GEOJSON_POINTS_PATH, SiteData.EMPTY, OptionalDouble.empty());
    assertEquals(3, sitesPoints.size());

    List<Site> sitesPolygon = Sites.fromGeoJson(
        GEOJSON_POLYGON_PATH, SiteData.EMPTY, OptionalDouble.empty());
    assertEquals(11654, sitesPolygon.size());
  }

  static final String TO_STRING =
      "Sites [size=6]\n" +
          "               Hilo HI                     -155.060 19.700 vs30=760.0 z1.0=NaN z2.5=NaN zSed=NaN\n" +
          "               Salt Lake City UT           -111.900 40.750 vs30=530.0 z1.0=0.3 z2.5=3.0 zSed=NaN\n" +
          "               San Jose CA                 -121.900 37.350 vs30=350.0 z1.0=NaN z2.5=NaN zSed=NaN\n" +
          "               Seattle WA                  -122.300 47.600 vs30=760.0 z1.0=1.2 z2.5=6.7 zSed=NaN\n" +
          "               Memphis TN                  -90.050 35.150 vs30=3000.0 z1.0=NaN z2.5=NaN zSed=NaN\n" +
          "               ... and 1 more ...";

}
