package gov.usgs.earthquake.nshmp.data;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;

class IntervalDataTests {

  static final double MIN = 2.0;
  static final double MAX = 6.0;
  static final double Δ = 1.0;
  static final double[] KEYS = { 2.5, 3.5, 4.5, 5.5 };

  @Test
  final void keysTest() {
    double[] actual = IntervalData.keys(MIN, MAX, Δ);
    assertArrayEquals(KEYS, actual, 0);
  }

  @Test
  final void indexOfTest() {
    int expected = 3;
    int actual = IntervalData.indexOf(MIN, Δ, KEYS[expected], KEYS.length);
    assertEquals(expected, actual, 0);

    assertThrows(IndexOutOfBoundsException.class, () -> {
      IntervalData.indexOf(MIN, Δ, 1.0, KEYS.length);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      IntervalData.indexOf(MIN, Δ, KEYS[0], -1);
    });
  }

  @Test
  final void checkDataStateTests() {
    assertThrows(IllegalStateException.class, () -> {
      IntervalData.checkDataState(null);
    });

    assertThrows(IllegalStateException.class, () -> {
      IntervalData.checkDataState(KEYS, null);
    });

    assertThrows(IllegalStateException.class, () -> {
      IntervalData.checkDataState(KEYS, KEYS, null);
    });
  }

}
