package gov.usgs.earthquake.nshmp.geo;

import static gov.usgs.earthquake.nshmp.Text.NEWLINE;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertNotSame;
import static org.junit.jupiter.api.Assertions.assertSame;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;

import gov.usgs.earthquake.nshmp.Maths;

class LocationListTests {

  private static LocationList locs1, locs2;
  private static Location p1, p2, p3, p4, p5, p6, p7;
  private static Location pp1, pp2;

  // private static Location g00, g01, g02, g03, g10, g11, g12, g13;
  // static LocationGrid grid;

  @BeforeAll
  static void setUp() {
    p1 = Location.create(0, -5);
    p2 = Location.create(-2, -3);
    p3 = Location.create(-2, -2);
    p4 = Location.create(0, 0);
    p5 = Location.create(2, 2);
    p6 = Location.create(2, 3);
    p7 = Location.create(0, 5);

    pp1 = Location.create(0, 0);
    pp2 = Location.create(1, 1);

    locs1 = LocationList.of(p1, p2, p3, p4, p5, p6, p7);
    locs2 = LocationList.of(p1, p3, p2, p4, p6, p5, p7);

    // g00 = Location.create(0, 0);
    // g01 = Location.create(1, 0);
    // g02 = Location.create(2, 0);
    // g03 = Location.create(3, 0);
    // g10 = Location.create(0, 1);
    // g11 = Location.create(1, 1);
    // g12 = Location.create(2, 1);
    // g13 = Location.create(3, 1);
    // grid = LocationGrid.builder(2, 4)
    // .fillRow(0, LocationList.builder().add(g00, g01, g02, g03).build())
    // .fillRow(1, LocationList.builder().add(g10, g11, g12, g13).build())
    // .build();
  }

  /* Factory */

  @Test
  final void create() {
    // check supplied is returned
    LocationList locs = LocationList.of(p1, p2, p3, p4, p5, p6, p7);
    assertEquals(locs, locs1);
    assertSame(locs, LocationList.copyOf(locs));
    // check supplied is copied
    List<Location> modifiableList = new ArrayList<>();
    modifiableList.addAll(locs1);
    assertEquals(modifiableList, locs1);
  }

  @Test
  final void createExceptions() {
    assertThrows(NullPointerException.class, () -> {
      Location[] locs = null;
      LocationList.of(locs);
    });

    assertThrows(IllegalStateException.class, () -> {
      Location[] locs = new Location[] {};
      LocationList.of(locs);
    });

    assertThrows(NullPointerException.class, () -> {
      List<Location> locs = null;
      LocationList.copyOf(locs);
    });

    assertThrows(IllegalStateException.class, () -> {
      List<Location> locs = ImmutableList.of();
      LocationList.copyOf(locs);
    });
  }

  @Test
  final void bounds() {
    Location min = Location.create(-2, -5);
    Location max = Location.create(2, 5);
    Bounds bounds = locs1.bounds();
    assertEquals(min, bounds.min);
    assertEquals(max, bounds.max);
  }

  @Test
  final void split() {
    Location loc1 = Location.create(0, 0);
    Location loc2 = Location.create(0.5, 0);
    Location loc3 = Location.create(1, 0);
    LocationList locList = LocationList.of(loc1, loc2, loc3);
    List<LocationList> subLists = locList.split(11);

    LocationList sl0 = subLists.get(0);
    assertEquals(sl0.first(), loc1);

    // Revisit: this changed when we removed
    // 5 decimal rounding in Locations

    // assertEquals(sl0.last(), Location.create(0.1, 0.0));
    assertEquals(sl0.last(), Location.create(0.09999999999999999, 6.1232308869976715E-18));

    LocationList sl3 = subLists.get(3);
    // assertEquals(sl3.first(), Location.create(0.3, 0.0));
    assertEquals(sl3.first(), Location.create(0.3, 1.8369664682372418E-17));

    // assertEquals(sl3.last(), Location.create(0.4, 0.0));
    assertEquals(sl3.last(), Location.create(0.39999999999999997, 2.4492867590777903E-17));

    LocationList sl6 = subLists.get(6);
    // assertEquals(sl6.first(), Location.create(0.6, 0.0));
    assertEquals(sl6.first(), Location.create(0.6000000000000001, 6.123230886997674E-18));
    // assertEquals(sl6.last(), Location.create(0.7, 0.0));
    assertEquals(sl6.last(), Location.create(0.7, 1.2246452447783745E-17));

    LocationList sl9 = subLists.get(9);
    // assertEquals(sl9.first(), Location.create(0.9, 0.0));
    assertEquals(sl9.first(), Location.create(0.8999999999999999, 2.449286759077791E-17));
    assertEquals(sl9.last(), loc3);

    // length greater than list
    subLists = locList.split(200);
    assertSame(locList, subLists.get(0));

    // undershoot
    Location loc4 = Location.create(1.00000000001, 0);
    locList = LocationList.of(loc1, loc4);
    subLists = locList.split(11);
  }

  @Test
  final void splitExceptions() {
    assertThrows(IllegalArgumentException.class, () -> {
      LocationList.of(pp1, pp2).split(Double.NaN);
    });
    assertThrows(IllegalArgumentException.class, () -> {
      LocationList.of(pp1, pp2).split(-1);
    });
  }

  @Test
  final void resample() {
    LocationList locs = LocationList.of(pp1, pp2);
    LocationList resampled = locs.resample(12);
    assertEquals(resampled.size(), 15);
    assertSame(resampled.first(), pp1);
    assertSame(resampled.last(), pp2);
    Location mid = resampled.get(7);

    // Revisit: this changed when we removed
    // 5 decimal rounding in Locations
    assertEquals(0.49996509384838933, mid.longitude, 0.0); // was 0.49997
    assertEquals(0.5000222122727477, mid.latitude, 0.0); // 0.50002

    // singleton
    locs = LocationList.of(pp1);
    resampled = locs.resample(1.0);
    assertSame(locs, resampled);
  }

  @Test
  final void resampleExceptions() {
    assertThrows(IllegalArgumentException.class, () -> {
      LocationList locs = LocationList.of(pp1, pp2);
      locs.resample(Double.POSITIVE_INFINITY);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      LocationList locs = LocationList.of(pp1, pp2);
      locs.resample(-1.0);
    });
  }

  public static void main(String[] args) {

    Location pp1 = Location.create(0, 0);
    Location pp2 = Location.create(1, 0);

    List<LocationList> partitions = LocationList.of(pp1, pp2).split(11);
    System.out.println(partitions);
    System.out.println(partitions.size());
    //
    // LocationList ll1 = null;
    // LocationList ll2 = LocationList.create(pp1, pp2);
    //
    // assertEquals(ll1, ll2);
    //
    // LocationList locs = LocationList.create(pp1, pp2);
    // LocationVector v = LocationVector.create(135 * GeoTools.TO_RAD, 5.0,
    // 5.0);
    // LocationList translated = locs.translate(v);
    // System.out.println(translated);
    //
    // System.out.println("---");
    // System.out.println(translated.get(0).lat());
    // System.out.println(translated.get(0).lon());
    // System.out.println(translated.get(0).depth());
    // System.out.println("---");
    // System.out.println(translated.get(1).lat());
    // System.out.println(translated.get(1).lon());
    // System.out.println(translated.get(1).depth());
    // System.out.println("---");

    // LocationList locs = LocationList.create(pp1, pp2);
    // LocationList resampled = locs.resample(12);
    // System.out.println(resampled);

    // List<Location> pp1 = ImmutableList.of(p1, p2, p3);
    // Iterable<Location> pp2 = LocationList.create(p1, p2, p3);
    // Iterable<Location> pp3 = LocationList.create(p1, p2, p3);
    // System.out.println(pp3.equals(pp2));

    // String locStr = new StringBuilder(NEWLINE)
    // .append("0.00000,0.00000,0.00000").append(NEWLINE)
    // .append("1.00000,1.00000,0.00000").append(NEWLINE)
    // .toString();
    // LocationList locs = LocationList.create(pp1, pp2);
    //
    // System.out.println("----");
    // System.out.println(locStr);
    // System.out.println("----");
    // System.out.println(locs);
    // System.out.println("----");

    // List<Integer> pp1 = Lists.newArrayList(2, 4, 7, 31);
    // List<Integer> pp2 = ImmutableList.of(2, 4, 7, 31);
    // System.out.println(pp1.hashCode());
    // System.out.println(pp2.hashCode());

  }

  @Test
  final void reverse() {

    // this tests the default RegularLocationList
    LocationList reversed = locs1.reverse();
    assertSame(locs1.get(0), reversed.get(6));
    assertSame(locs1.get(1), reversed.get(5));
    assertSame(locs1.get(2), reversed.get(4));
    assertSame(locs1.get(4), reversed.get(2));
    assertSame(locs1.get(5), reversed.get(1));
    assertSame(locs1.get(6), reversed.get(0));
    assertNotSame(reversed, locs1);

    // use LocationGrid to test fall through to
    // other LocList implementations
    // LocationList reversedRow = grid.row(1).reverse();
    // assertSame(g10, reversedRow.get(3));
    // assertSame(g11, reversedRow.get(2));
    // assertSame(g12, reversedRow.get(1));
    // assertSame(g13, reversedRow.get(0));
  }

  @Test
  final void translate() {
    LocationList locs = LocationList.of(pp1, pp2);
    LocationVector v = LocationVector.create(135 * Maths.TO_RADIANS, 5.0, 5.0);
    LocationList transLoc = locs.translate(v);
    // Location pp1trans = Location.create(0.0318, -0.0318, 5.0);
    // Location pp2trans = Location.create(1.0318, 0.9682, 5.0);

    Location pp1trans = Location.create(0.031795787631496104, -0.03179578273558637, 5.0);
    Location pp2trans = Location.create(1.031800322985746, 0.9682040632704144, 5.0);

    // Revisit: this changed when we removed
    // 5 decimal rounding in Locations
    assertEquals(pp1trans, transLoc.get(0));
    assertEquals(pp2trans, transLoc.get(1));
  }

  @Test
  final void fromString() {
    String locStr = "-117.0,34.0,0.1 -117.0,34.1,0.2 -117.1,34.0,0.3";
    LocationList locsFromString = LocationList.fromString(locStr);
    LocationList locsActual = LocationList.of(
        Location.create(-117.0, 34.0, 0.1),
        Location.create(-117.0, 34.1, 0.2),
        Location.create(-117.1, 34.0, 0.3));
    assertEquals(locsFromString, locsActual);
  }

  /* Object */

  @Test
  final void hashCodeTest() {
    // copy create should return the same list
    LocationList copy = LocationList.copyOf(locs1);
    assertEquals(copy.hashCode(), locs1.hashCode());
    // using builder should copy locations
    copy = LocationList.builder().addAll(locs1).build();
    assertEquals(copy.hashCode(), locs1.hashCode());
    assertNotEquals(locs1.hashCode(), locs2.hashCode());
    // because hash code is based on backing list
    // implementation, check that too
    List<Location> locs = ImmutableList.copyOf(locs1);
    assertEquals(locs.hashCode(), locs1.hashCode());

    // hash codes for concrete and view lists??
  }

  @Test
  final void equalsTest() {
    // some of these predate delegation of equals to backing
    // List implementation

    assertEquals(locs1, locs1);
    LocationList equal = null;
    assertNotEquals(locs1, equal);
    assertNotEquals(locs1, "test");
    equal = LocationList.of(p1, p2, p3, p4, p5, p6, p7);
    assertEquals(locs1, equal);
    assertNotEquals(locs1, locs2);

    // size check
    equal = LocationList.of(p1, p2, p3, p4, p5);
    assertNotEquals(locs1, equal);

    // check that ImmutableList is not equal to LocationList of same Locations
    List<Location> equalList = ImmutableList.of(p1, p2, p3, p4, p5, p6, p7);
    assertNotEquals(locs1, equalList);
  }

  @Test
  final void toStringTest() {
    String locStr = new StringBuilder(NEWLINE)
        .append("0.0,0.0,0.0").append(NEWLINE)
        .append("1.0,1.0,0.0").append(NEWLINE)
        .toString();
    LocationList locs = LocationList.of(pp1, pp2);
    assertEquals(locStr, locs.toString());
  }

  /* Methods */

  @Test
  final void size() {
    assertEquals(locs1.get(0), locs1.first());
  }

  @Test
  final void get() {
    assertSame(locs1.get(0), p1);
    assertSame(locs1.get(2), p3);
    assertSame(locs1.get(4), p5);
    assertSame(locs1.get(6), p7);
  }

  @Test
  final void first() {
    assertSame(locs1.get(0), locs1.first());
  }

  @Test
  final void last() {
    assertSame(locs1.get(locs1.size() - 1), locs1.last());
  }

  @Test
  final void length() {
    assertEquals(1479.6049574653778, locs1.length(), 0.0);
    assertEquals(157.25055720494782, LocationList.of(pp1, pp2).length(), 0.0);
    assertEquals(0.0, LocationList.of(p1).length(), 0.0);
  }

  @Test
  final void depth() {
    LocationList locs = LocationList.of(
        Location.create(0, 0, 0),
        Location.create(1, 1, 0),
        Location.create(2, 2, 1));
    assertEquals(0.3333333333333333, locs.depth(), 0.0);
  }

  /* Builder */

  @Test
  final void builder() {
    LocationList.Builder b = LocationList.builder();

    LocationList locs = b.add(p1).build();
    assertEquals(1, locs.size());
    assertSame(locs.get(0), p1);

    double lon = -117.044;
    double lat = 34.001;
    locs = b.add(lon, lat).build();
    assertEquals(2, locs.size());
    assertEquals(locs.get(1), Location.create(lon, lat));

    double depth = 3.2;
    locs = b.add(lon, lat, depth).build();
    assertEquals(3, locs.size());
    assertEquals(locs.get(2), Location.create(lon, lat, depth));

    locs = b.add(p1, p2, p3).build();
    assertEquals(6, locs.size());
    assertSame(locs.get(5), p3);

    List<Location> locList = Lists.newArrayList(p4, p5, p6);
    locs = b.addAll(locList).build();
    assertEquals(9, locs.size());
    assertSame(locs.get(8), p6);
  }

  @Test
  final void builderAddExceptions() {
    assertThrows(NullPointerException.class, () -> {
      Location loc = null;
      LocationList.builder().add(loc).build();
    });

    assertThrows(NullPointerException.class, () -> {
      Location loc = null;
      LocationList.builder().add(Location.create(0, 0), loc).build();
    });

    assertThrows(NullPointerException.class, () -> {
      List<Location> locs = Lists.newArrayList(
          Location.create(0, 0), null);
      LocationList.builder().addAll(locs).build();
    });

    assertThrows(IllegalStateException.class, () -> {
      LocationList.builder().build();
    });
  }

}
