package gov.usgs.earthquake.nshmp.gmm;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;

import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

class InstanceTest {

  /*
   * Intantiates all supported IMTs for all GMMs and runs a test calculation.
   */

  private static GmmInput GMM_INPUT = GmmInput.builder().withDefaults().build();

  @ParameterizedTest
  @MethodSource("gmmImtPairs")
  public void testInstances(Gmm gmm, Imt imt) {
    assertDoesNotThrow(() -> {
      GroundMotionModel instance = gmm.instance(imt);
      instance.calc(GMM_INPUT);
    });
  }

  private static Stream<Arguments> gmmImtPairs() {
    List<Arguments> argsList = new ArrayList<>();
    Set<Gmm> gmms = EnumSet.allOf(Gmm.class);
    for (Gmm gmm : gmms) {
      Set<Imt> imts = gmm.supportedImts();
      for (Imt imt : imts) {
        argsList.add(Arguments.of(gmm, imt));
      }
    }
    return argsList.stream();
  }
}
