package gov.usgs.earthquake.nshmp.gmm;

import static gov.usgs.earthquake.nshmp.gmm.Gmm.AS_97_AK;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.BJF_97;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.CB_03;
import static gov.usgs.earthquake.nshmp.gmm.Imt.PGA;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA0P2;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA1P0;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA2P0;

import java.io.IOException;
import java.util.EnumSet;
import java.util.Set;
import java.util.stream.Stream;

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.ArgumentsProvider;
import org.junit.jupiter.params.provider.ArgumentsSource;

class LegacySoftRock implements ArgumentsProvider {

  /*
   * Tests for older active crust ground motion models that have limited
   * parameterizations and only support soft-rock sites. These models are
   * currently used in the 2007 Alaska NSHM.
   */

  private static String GMM_INPUTS = "/gmm/ceus-vs760-inputs.csv";
  private static String GMM_RESULTS = "/gmm/legacy-vs760-results.csv";

  @ParameterizedTest(name = "({index}) {0} {2} {1}")
  @ArgumentsSource(LegacySoftRock.class)
  void test(int index, Gmm gmm, Imt imt, double exMedian, double exSigma, String inputs) {
    GmmTest.test(index, gmm, imt, exMedian, exSigma, inputs);
  }

  @Override
  public Stream<? extends Arguments> provideArguments(ExtensionContext context) throws Exception {
    return GmmTest.loadResults(GMM_RESULTS, GMM_INPUTS);
  }

  /* Result generation sets */
  private static Set<Gmm> gmms = EnumSet.of(AS_97_AK, BJF_97, CB_03);
  private static Set<Imt> imts = EnumSet.of(PGA, SA0P2, SA1P0, SA2P0);

  public static void main(String[] args) throws IOException {
    GmmTest.generateResults(gmms, imts, GMM_INPUTS, GMM_RESULTS);
  }

}
