package gov.usgs.earthquake.nshmp.gmm;

import static gov.usgs.earthquake.nshmp.gmm.Gmm.USGS_PRVI_ACTIVE_CRUST;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.USGS_PRVI_ACTIVE_CRUST_ADJUSTED;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.USGS_PRVI_ACTIVE_CRUST_COMBINED_TREE;

import java.io.IOException;
import java.util.EnumSet;
import java.util.Set;
import java.util.stream.Stream;

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.ArgumentsProvider;
import org.junit.jupiter.params.provider.ArgumentsSource;

public class UsgsPrviBackboneCrustal_tests implements ArgumentsProvider {

  private static String GMM_INPUTS = "/gmm/nga-west-inputs.csv";
  private static String GMM_RESULTS = "/gmm/usgs-prvi-backbone-crustal-gmms-results.csv";

  @ParameterizedTest(name = "({index}) {0} {2} {1}")
  @ArgumentsSource(UsgsPrviBackboneCrustal_tests.class)
  void test(int index, Gmm gmm, Imt imt, double exMedian, double exSigma, String inputs) {
    GmmTest.test(index, gmm, imt, exMedian, exSigma, inputs);
  }

  @Override
  public Stream<? extends Arguments> provideArguments(ExtensionContext context) throws Exception {
    return GmmTest.loadResults(GMM_RESULTS, GMM_INPUTS);
  }

  /* Result generation sets */
  private static Set<Gmm> gmms = EnumSet.of(
      USGS_PRVI_ACTIVE_CRUST,
      USGS_PRVI_ACTIVE_CRUST_ADJUSTED,
      USGS_PRVI_ACTIVE_CRUST_COMBINED_TREE);

  private static Set<Imt> imts = UsgsPrviBackbone2025.COEFFS_DATA_ADJUSTMENT.imts();

  public static void main(String[] args) throws IOException {
    GmmTest.generateResults(gmms, imts, GMM_INPUTS, GMM_RESULTS);
    System.out.println("Done");
  }
}
