package gov.usgs.earthquake.nshmp.tree;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.Test;

class TreesTests {

  @Test
  void transposeAndTransform() {

    LogicTree<Integer> tree1 = LogicTree.<Integer> builder("test-tree-1")
        .addBranch("B1", 1, 0.2)
        .addBranch("B2", 4, 0.5)
        .addBranch("B3", 7, 0.3)
        .build();

    LogicTree<Integer> tree2 = LogicTree.<Integer> builder("test-tree-2")
        .addBranch("B1", 2, 0.2)
        .addBranch("B2", 5, 0.5)
        .addBranch("B3", 8, 0.3)
        .build();

    LogicTree<Integer> tree3 = LogicTree.<Integer> builder("test-tree-3")
        .addBranch("B1", 3, 0.2)
        .addBranch("B2", 6, 0.5)
        .addBranch("B3", 9, 0.3)
        .build();

    List<LogicTree<Integer>> listOfTrees = List.of(tree1, tree2, tree3);
    LogicTree<List<Integer>> treeOfLists = Trees.transpose(listOfTrees);

    assertEquals(List.<Integer> of(1, 2, 3), treeOfLists.get(0).value());
    assertEquals(List.<Integer> of(4, 5, 6), treeOfLists.get(1).value());
    assertEquals(List.<Integer> of(7, 8, 9), treeOfLists.get(2).value());

    LogicTree<Integer> transformed = Trees.transform(
        treeOfLists,
        list -> list.stream().mapToInt(Integer::intValue).sum(),
        Optional.of("transformed-tree"));

    assertEquals(6, transformed.get(0).value());
    assertEquals(15, transformed.get(1).value());
    assertEquals(24, transformed.get(2).value());
  }

}
